@@ -31,6 +31,7 @@ class Facet(object):
         self.template = template or self.template
         self.advanced_template = advanced_template or self.advanced_template
         self.description = description
+        self.login_required = kwargs.pop('login_required', False)
 
         default_related_column_name = self.field.split('.')[0]
         related_column_name = kwargs.get('related_column_name')
@@ -361,8 +362,8 @@ class RangeFilter(Facet):
         # This is a list of filter query objects for each value range.
         filters = []
 
-        # if key is equal to _missing, we create a query that returns every document that doesn't have a value for that field.
-        if key == '_missing':
+        # if key is equal to _missing or empty string, we create a query that returns every document that doesn't have a value for that field.
+        if key in ['_missing', ""]:
             filters.append(~Q('exists', field=self.field))
         else:
             # This function will only return filters for ranges that are defined in self.ranges (i.e - valid ranges)
